#!/usr/bin/env python
#
# Data API:
# http://api.stackexchange.com/docs/questions#page=1&order=desc&min=2&sort=votes&tagged=application-development&filter=default&site=askubuntu&run=true
#

import os
import json
import time

from urllib2 import urlopen, Request
from zlib import decompress, MAX_WBITS


class Cookbook:
    
    def __init__(self, name, sections, tags, exclude):
        self.name = name
        self.MIN_VOTES = 2
        self.API_ROOT = 'http://api.stackexchange.com/2.1'
        self.TAGS = tags
        self.EX_TAGS = exclude or set([])
        self.SITE = 'askubuntu'
        self.PAGE = 1

        self.HTML_OUT = os.path.join(os.path.dirname(__file__), self.name+'.html')
        self.JSON_OUT = os.path.join(os.path.dirname(__file__), self.name+'.json')
        self.SECTIONS = sections


        self.current_run = {
            'keep': list(),
            'data': dict(),
            'skip': list(),
            'sections': dict(),
        }

def load_previous(cookbook):
    if os.path.exists(cookbook.JSON_OUT):
        json_fh = open(cookbook.JSON_OUT)
        cookbook.current_run = json.loads(json_fh.read())
        json_fh.close()
    else:
        print "No file at %s" % cookbook.JSON_OUT

def save_run(cookbook):
    json_fh = open(cookbook.JSON_OUT, 'w')
    json_fh.write(json.dumps(cookbook.current_run))
    json_fh.close()
    
def build_query(cookbook):
    q = [
        'pagesize=100',
        'order=desc',
        'min=%s' % cookbook.MIN_VOTES,
        'sort=votes',
        'tagged=%s' % ';'.join(cookbook.TAGS), 
        'filter=!9hnGsreuM',
         'page=%s' % cookbook.PAGE,
        ]
    return '&'.join(q)

def site(url_provider):
    def wrapper(cookbook):
        url = url_provider(cookbook)
        if '?' in url:
            return url + '&site=' + cookbook.SITE
        else:
            return url + '?site=' + cookbook.SITE
    return wrapper
    
@site
def build_questions_url(cookbook):
    return '%s/questions?%s' % (cookbook.API_ROOT, build_query(cookbook))
@site
def build_details_url(cookbook, question_id):
    return '%s/questions/%s' % (cookbook.API_ROOT, question_id)

def fetch_contents(cookbook):
    contents = []
    has_more = True

    while has_more:
        req = Request(build_questions_url(cookbook))
        resp = urlopen(req).read()
        results = json.loads(decompress(resp, 16 + MAX_WBITS).decode('UTF-8'))
        contents = contents + results['items']
        if results['quota_remaining'] < 5:
            raise Exception('Quota too low to continue. Please wait')

        if 'backoff' in results:
            time.sleep(data['backoff'] + 2)

        has_more = results['has_more']
        cookbook.PAGE += 1

    return contents

def show_question(cookbook, question):
    answer = None
    print '(%(score)s) %(title)s' % question
    while not answer:
        answer = raw_input('[Y/n/m/t/r]? ')
        if answer and answer[0].lower() == 'y':
            set_section(cookbook, question)
            cookbook.current_run['data'][str(question['question_id'])] = {'id': question['question_id'], 'title': question['title'], 'link': question['link']}
            cookbook.current_run['keep'].append(question['question_id'])
        elif answer and answer[0].lower() == 'n':
            cookbook.current_run['skip'].append(question['question_id'])
            break
        elif answer and answer[0].lower() == 'm':
            print question['body']
            answer = None
        elif answer and answer[0].lower() == 't':
            print ', '.join(question['tags'])
            answer = None
        elif answer and answer[0].lower() == 'r':
            print '\n(%(score)s) %(title)s' % question
            answer = None
        elif answer and answer[0].lower() == 'i':
            break
        elif answer and answer[0].lower() == 'o':
            os.system('xdg-open %s' % question['link'])
            answer = None
        elif answer and answer[0].lower() == 'q':
            quit_run(cookbook)
        elif answer and answer[0].lower() == '?':
            print '''
Y = Yes, use this question
n = No, skip this question
m = More info, show the question body
t = Show tags
r = Repeat the question
i = Ignore the question for now
o = Open question in a browser
q = Quit without saving
            ''' % question
            answer = None
        elif answer:
            print '\n Unknown option: \'%s\', enter \'?\' for options' % answer
            answer = None
    
def set_section(cookbook, question):
    answer = None
    print 'What section?'
    while not answer:
        answer = raw_input('[%s]? ' % '/'.join(cookbook.SECTIONS.keys()))
        if answer and answer[0].lower() == '?':
            for section_char, section in cookbook.SECTIONS.items():
                print '%s =  %s' % (section_char, section['title'])
            answer = None
        elif answer and answer[0].lower() == 'q':
            quit_run(cookbook)
        elif answer and answer[0].lower() in cookbook.SECTIONS:
            if answer[0].lower() not in cookbook.current_run['sections']:
                cookbook.current_run['sections'][answer[0].lower()] = list()
            cookbook.current_run['sections'][answer[0].lower()].append(question['question_id'])
        elif answer:
            print '\n Unknown option: \'%s\', enter \'?\' for options' % answer
            answer = None
    
def quit_run(cookbook):
    save_run(cookbook)
    html_fh = open(cookbook.HTML_OUT, 'w')
    for section_char, section in cookbook.SECTIONS.items():
        if section_char in cookbook.current_run['sections']:
            html_fh.write('<h2>%s</h2>\n<ul>\n' % section['title'])
            for question_id in cookbook.current_run['sections'][section_char]:
                question = cookbook.current_run['data'][str(question_id)]
                html_fh.write('    <li><a href="%(link)s" target="_blank">%(title)s</a></li>\n' % question)
            html_fh.write('</ul>\n')
    html_fh.write('<div class="box-orange clearfix"><h3 style="margin: 0 0 8px 0; padding-top: 0; border-top: none;">Still have questions?</h3><a class="link-action-small" href="http://www.askubuntu.com/questions/ask?tags='+','.join(cookbook.TAGS)+'" target="_blank"><span class="cta-left"></span><span class="cta-center">Ask your Question</span><span class="cta-right"></span></a><span>If you could not find an answer to your question here, you can ask the thousands of Ubuntu users and developers on Ask Ubuntu.</span></div>\n\n')
    html_fh.close()
    exit(0)
    
def build_cookbook(cookbook):
    data = fetch_contents(cookbook)

    load_previous(cookbook)
    alltags = dict()
    for question in data:
        if question['is_answered'] and not 'closed_date' in question:
            if len(cookbook.EX_TAGS & set(question['tags'])) > 0:
                continue
            if question['question_id'] in cookbook.current_run['keep']:
                # Update the question data in case it was changed on AU
                cookbook.current_run['data'][str(question['question_id'])] = {'id': question['question_id'], 'title': question['title'], 'link': question['link']}
                continue
            if question['question_id'] in cookbook.current_run['skip']:
                continue
                
            show_question(cookbook, question)
    quit_run(cookbook)
